import gradio as gr
import os
import time
import re
import torch
from transformers import AutoModelForCausalLM, AutoTokenizer
import argparse

# Argument parser for command-line options
parser = argparse.ArgumentParser(description='Gradio Web UI for Image Captioning')
parser.add_argument('--bit4', action='store_true', help='Use the Qwen/Qwen-VL-Chat-Int4 model for 4-bit quantization')
parser.add_argument('--bf16', action='store_true', help='Use bf16 precision')
parser.add_argument('--fp16', action='store_true', help='Use fp16 precision')
parser.add_argument('--share', action='store_true', help='share')
args = parser.parse_args()

# Choose the model based on the --4bit flag
model_name = "Qwen/Qwen-VL-Chat-Int4" if args.bit4 else "Qwen/Qwen-VL-Chat"

# Initialize the model and tokenizer with dynamic precision setting
tokenizer = AutoTokenizer.from_pretrained(model_name, trust_remote_code=True)
model_kwargs = {
    'device_map': "cuda",
    'trust_remote_code': True,
    'use_flash_attn': True,
	'bf16' : True,
	'fp16' : False
}
if args.bf16:
    model_kwargs['bf16'] = True
    print("bf16 loaded")
elif args.fp16:
    model_kwargs['bf16'] = False
    model_kwargs['fp16'] = True
    print("fp16 loaded")
model = AutoModelForCausalLM.from_pretrained(model_name, **model_kwargs).eval()

# Function to check for unwanted elements in the caption
def has_unwanted_elements(caption):
    patterns = [r'<ref>.*?</ref>', r'<box>.*?</box>']
    return any(re.search(pattern, caption) for pattern in patterns)

# Function to clean up the caption
def clean_caption(caption):
    caption = re.sub(r'<ref>(.*?)</ref>', r'\1', caption)
    caption = re.sub(r'<box>.*?</box>', '', caption)
    return caption.strip()

# Function for single image captioning
def caption_image(image_path, prompt):
    # The 'image_path' parameter directly contains the path to the uploaded image file
    
    query = tokenizer.from_list_format([
        {'image': image_path},  # Directly use the image path
        {'text': prompt},
    ])
    response, _ = model.chat(tokenizer, query=query, history=None)
    
    if has_unwanted_elements(response):
        response = clean_caption(response)
    
    return response

# Function for batch image captioning
def batch_caption_images(imgdir, prompt):
    image_types = ['.png', '.jpg', '.jpeg', '.bmp', '.gif']
    files = [f for f in os.listdir(imgdir) if os.path.splitext(f)[1].lower() in image_types]
    total_files = len(files)
    processed_count = 0
    
    start_time = time.time()
    
    for filename in files:
        file_start_time = time.time()
        
        image_path = os.path.join(imgdir, filename)
        query = tokenizer.from_list_format([
            {'image': image_path},
            {'text': prompt},
        ])
        response, _ = model.chat(tokenizer, query=query, history=None)
        
        if has_unwanted_elements(response):
            response = clean_caption(response)
        
        caption_file_path = os.path.join(imgdir, os.path.splitext(filename)[0] + '.txt')
        with open(caption_file_path, 'w', encoding='utf-8') as caption_file:
            caption_file.write(response)
        
        processed_count += 1
        file_end_time = time.time()
        
        print(f"Processed {processed_count}/{total_files}.")
        print(f"Caption for {filename} generated in {file_end_time - file_start_time:.2f} seconds.")
        
        if processed_count < total_files:
            estimated_time_left = (file_end_time - start_time) / processed_count * (total_files - processed_count)
            print(f"Estimated time to complete: {estimated_time_left:.2f} seconds.")
        else:
            print("Batch processing complete.")

# Gradio app setup
with gr.Blocks() as app:
    with gr.Row():
        with gr.Column():
            image_upload = gr.Image(type="filepath", label="Upload Image")
        with gr.Column():
            prompt_input = gr.Textbox(value="describe the image like a blip model but make it super detailed and include image drawing style if it is 3d, render, cartoon, anime and such", label="Caption Prompt")
            generate_caption_btn = gr.Button("Generate Caption")
            caption_output = gr.Textbox(label="Caption Result")
    
    with gr.Row():
        imgdir_input = gr.Textbox(label="Images Directory")
        batch_caption_btn = gr.Button("Batch Caption Images")
        batch_output = gr.Textbox(label="Batch Caption Results")
    
    generate_caption_btn.click(caption_image, inputs=[image_upload, prompt_input], outputs=caption_output)
    batch_caption_btn.click(batch_caption_images, inputs=[imgdir_input, prompt_input], outputs=batch_output)

app.launch(share=args.share,inbrowser=True)
