import gradio as gr
import os
import re
import torch
from transformers import AutoModelForCausalLM, AutoTokenizer

# Initialize the model and tokenizer
tokenizer = AutoTokenizer.from_pretrained("Qwen/Qwen-VL", trust_remote_code=True)
model = AutoModelForCausalLM.from_pretrained("Qwen/Qwen-VL", device_map="cuda", trust_remote_code=True, use_flash_attn=True).eval()

# Function to check for unwanted elements in the caption
def has_unwanted_elements(caption):
    patterns = [r'<ref>.*?</ref>', r'<box>.*?</box>']
    return any(re.search(pattern, caption) for pattern in patterns)

# Function to clean up the caption
def clean_caption(caption):
    caption = re.sub(r'<ref>(.*?)</ref>', r'\1', caption)
    caption = re.sub(r'<box>.*?</box>', '', caption)
    return caption.strip()

# Function for single image captioning
def caption_image(image_path, prompt):
    # The 'image_path' parameter directly contains the path to the uploaded image file
    
    query = tokenizer.from_list_format([
		{'image': image_path},
		{'text': prompt},
	])
    inputs = tokenizer(query, return_tensors='pt')
    inputs = inputs.to(model.device)
    pred = model.generate(**inputs)
    response = tokenizer.decode(pred.cpu()[0], skip_special_tokens=False)	

    
    if has_unwanted_elements(response):
        response = clean_caption(response)
    
    return response

# Function for batch image captioning
def batch_caption_images(imgdir, prompt):
    image_types = ['.png', '.jpg', '.jpeg', '.bmp', '.gif']
    files = [f for f in os.listdir(imgdir) if os.path.splitext(f)[1].lower() in image_types]
    captions = []
    for filename in files:
        image_path = os.path.join(imgdir, filename)
        query = tokenizer.from_list_format([
            {'image': image_path},
            {'text': prompt},
        ])
        response, _ = model.chat(tokenizer, query=query, history=None)
        
        if has_unwanted_elements(response):
            response = clean_caption(response)
        
        captions.append(f"{filename}: {response}")
    return "\n".join(captions)

# Gradio app setup
with gr.Blocks() as app:
    with gr.Row():
        with gr.Column():
            image_upload = gr.Image(type="filepath", label="Upload Image")
        with gr.Column():
            prompt_input = gr.Textbox(value="Just caption the image with details, colors, items, objects, emotions, art style, drawing style and objects but do not add any description or comment. Do not miss any item in the given image", label="Caption Prompt")
            generate_caption_btn = gr.Button("Generate Caption")
            caption_output = gr.Textbox(label="Caption Result")
    
    with gr.Row():
        imgdir_input = gr.Textbox(label="Images Directory")
        batch_caption_btn = gr.Button("Batch Caption Images")
        batch_output = gr.Textbox(label="Batch Caption Results")
    
    generate_caption_btn.click(caption_image, inputs=[image_upload, prompt_input], outputs=caption_output)
    batch_caption_btn.click(batch_caption_images, inputs=[imgdir_input, prompt_input], outputs=batch_output)

app.launch(share=True)
